<?php

namespace Drupal\Tests\xero\Unit;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\serialization\Normalizer\ComplexDataNormalizer;
use Drupal\serialization\Normalizer\TypedDataNormalizer;
use Drupal\Tests\UnitTestCase;
use Drupal\Tests\xero\Traits\XeroGuidTrait;
use Drupal\xero\XeroQuery;
use Drupal\xero\Normalizer\XeroNormalizer;
use Prophecy\PhpUnit\ProphecyTrait;
use Symfony\Component\Serializer\Serializer;

/**
 * Class XeroQueryTestBase provides helper methods for unit tests.
 */
abstract class XeroQueryTestBase extends UnitTestCase {

  use ProphecyTrait;
  use XeroGuidTrait;

  /**
   * @var \Drupal\Core\TypedData\TypedDataManager
   */
  protected $typedDataManager;

  /**
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * @var \Psr\Log\LoggerInterface;
   */
  protected $loggerFactory;

  /**
   * @var \Radcliffe\Xero\XeroClient
   */
  protected $client;

  /**
   * @var \Drupal\Core\Cache\NullBackend
   */
  protected $cache;

  /**
   * @var \Drupal\xero\XeroQuery
   */
  protected $query;

  protected function setUp(): void {
    parent::setUp();

    // Setup a Null cache backend.
    $this->cache = $this->createMock('Drupal\Core\Cache\NullBackend');

    // Setup LoggerChannelFactory.
    $loggerProphet = $this->prophesize('\Psr\Log\LoggerInterface');
    $this->loggerFactory = new LoggerChannelFactory();
    $this->loggerFactory->addLogger($loggerProphet->reveal());

    // Mock the Typed Data Manager.
    $this->typedDataManager = $this->createMock('Drupal\Core\TypedData\TypedDataManager');

    // Mock XeroClient.
    $this->client = $this->createMock('\Radcliffe\Xero\XeroClient');

    // Setup the container.
    $container = new ContainerBuilder();
    $container->set('typed_data_manager', $this->typedDataManager);
    $container->set('serializer', $this->serializer);
    \Drupal::setContainer($container);

    // Setup the Serializer component class.
    $this->serializer = new Serializer([
      new ComplexDataNormalizer(),
      new XeroNormalizer($this->typedDataManager),
      new TypedDataNormalizer(),
    ]);

    $this->query = new XeroQuery($this->client, $this->serializer, $this->typedDataManager, $this->loggerFactory, $this->cache);
  }

}
